function [A_EBCS,sel_ind,A_EBCS_var] = networkRefinement...
    (A_pred,ss,C,os,y,initx,T,x_pred,noiseR,noiseQ,N_per,T1)
%
%This function is to Step3: network refinement
%
% OUTPUT ARGUMENTS:
% 
%	A_EBCS      inferred transition matrix from step3
% 
%	sel_ind     index of selected time-variance related rows of A in
%               refinement
%	A_EBCS_var  variance of inferred transition matrix from step3
% 

%
% INPUT ARGUMENTS:
% 
%	A_pred      inferred transition matrix from step2
% 
%	ss          total number of species
% 
%	C           measurement matrix
% 
%	os          number of combined measurements
% 
%	y           combined measurements
% 
%	initx      	initial concentrations of species in each perturbations
% 
%	T           total number of time points
% 
%	x_pred      inferred x from step1
% 
%	noiseR      assumed variance of measurement noise
% 
%	noiseQ      assumed variance of process noise
% 
%	N_per      	Number of perturbations
% 
%	T1      	number of time points in each perturbation

% create a matrix contains initial concentrations of x
x_init = [];
for j = 1:N_per
    xj = initx{j}*ones(1,T1);
    x_init = [x_init,xj];
end

% remove row j of A_pred that only have one non-zero entry in jth row and
% jth column; and this non-zero entry is close to 1
A_pred2 = round(A_pred*100)/100;
sel_metric = sum(abs(A_pred2)>0,2)==1&sum(abs(A_pred2)>0,1)'==1&sum(abs(A_pred2-1)<0.05,2)==1;
remove_ind = find(sel_metric);
sel_ind = find(~sel_metric);

C_new = zeros(os,ss);% updated C with time-invariant species related entries removed
y_new = zeros(os,T);% updated y with time-invariant species related entries removed
for i = 1:os
    C_i = C(i,:);% each row of C
    y_i = y(i,:); % each row of y
    measured_x_ind = find(C_i);% non-zero entries of the current row of C
    % for each non-zero entry, if it corresponds to time-invariant species
    % then this entry will be removed. Also the corresponding measurement
    % results y will be updated.
    for j=1:length(measured_x_ind)
        pos_j = measured_x_ind(j);
        if sum(remove_ind==pos_j)==1
               y_i = y_i-x_init(pos_j,:);
               C_i(pos_j) = 0;
        end
    end
    C_new(i,:) = C_i;
    y_new(i,:) = y_i;
end

% remove the row of C_new that all entries are zero
sel_C = find(sum(C_new,2)>0);
C_star = C_new(sel_C,sel_ind);

ss = length(sel_ind);% number of remained species
y_star = y_new(sel_C,:); % selected measurements
os = size(C_star,1); % number of updated measurements

R = noiseR*eye(os); % covariance matrix of measurement noise
Q = noiseQ*eye(ss);% covariance matrix of process noise

% linear Bayesian regression to infer x
x_pred_s = x_pred(sel_ind,:);% selected time-variant species
eta_inv = diag(Q);% diagonal of Q
mu_xt = cell(N_per,1);% refined x
sigma_xt = cell(N_per,1); % covariance of refined x
% for each perturbation infer x
for i =1:N_per
    x_pred1 = x_pred_s(:,T1*(i-1)+1:T1*i); % inferred x from step1 in each perturbation
    y1 = y_star(:,T1*(i-1)+1:T1*i); % updated measurements in each perturbation
    [mu_xt1,sigma_xt1] = xInference(ss,x_pred1,R,C_star,y1,T1);% infer x
    mu_xt{i} = mu_xt1; % save mean value of x
    sigma_xt{i} = sigma_xt1; % save covariance of x
end
[gamma1  beta] = xt_statistic(ss,mu_xt,sigma_xt,T1,N_per); % calculate some statistics of x
num_A_non_zero= ss*T;% number of entries in x

% network inference via extended Bayesian compressive sensing
[A_EBCS,A_EBCS_var] = extendBCS(ss,eta_inv,gamma1,beta);


