function [result x_all y_all] = pathway_inference_benchmark(N_per,A,ss,os,T,T1,genNewC,noiseR,noiseQ,x_all,y_all)
%
%This function is the main function to do pathway inference
%
% INPUT ARGUMENTS:
%
%	N_per	Number of perturbations
%
%	A		ss*ss transition matrix
%
%	ss		total number of species
%
%	os		total number of combined measurements
%
%	T       total number of time points
%
%   T1      number of time points in each perturbation
%
%   genNewC if ==1,then Low-density parity-check code will generate new C
%
%   noiseR  assumed variance of measurement noise
%
%   noiseQ  assumed variance of process noise
%
%   x_all   time-series of species concentrations. If it is empty, then
%           this program will generate it
%
%   y_all   time-series of combined concentrations. If it is empty, then
%           this program will generate it
%
% OUTPUT ARGUMENTS:
%
%   result  the inferred results
%
%   x_all   time-series of species concentrations used in this function
%
%   y_all   time-series of combined concentrations used in this function
%

%% initialise x
if isempty(x_all)
    initx = cell(N_per,1);
    x = [];
    x_all = cell(N_per,1);
    for n = 1:N_per
        initx{n} = normrnd(100,30,[1,ss])';% randomly generate the initial condition of x
        % generate x(t)
        x_n = zeros(ss, T1);
        x_n(:,1) = initx{n};
        for t=2:T1
            x_n(:,t) = A*x_n(:,t-1) ;
        end
        %
        x = [x,x_n];
        x_all{n} = x_n;
    end
else% used the input x_all
    x = [];
    initx = cell(N_per,1);
    for n = 1:N_per
        x = [x,x_all{n}];
        initx{n} = x_all{n}(:,1);
    end
end

%% generate measurement matrix
if genNewC ==1 % Low-density parity-check code is used to generate measurement matrix
    C_cand = double(makeLdpc(os, ss, 1, 1, 3));
    % check whether all species are covered in measurement matrix
    if sum(sum(C_cand,1)==0)>0 ||sum(sum(C_cand,2)==0)>0
        sprintf('C is not well generated')
    else
        sprintf('C generated')
    end
else % load the existing measurement matrix
    load C_cand.mat
end

%% step1 and 2: Concentration inference and Network inference via compressive sensing
C = C_cand;% measurement matrix
R = noiseR*eye(os);% covariance matrix of measurement noise
% generate y
if isempty(y_all)
    y = [];
    y_all = cell(N_per,1);
    for k = 1:N_per
        y_k = sample_lds_y(x_all{k}, C, R, T1);% generate combined measurement y in each perturbation
        y = [y,y_k];
        y_all{k} = y_k;
    end
else% used the input y_all
    y = [];
    for k = 1:N_per
        y = [y,y_all{k}];
    end
end

% use compressive sensing to infer concentrations and network
[A_own mu_xt] = delta_x_and_A_Inference(ss,initx,C,y,T,T1,N_per,noiseR);

% remove NaN entry in A_own
A_pred = A_own;
A_pred(isnan(A_pred)) = 0;

x_error = sqrt(sum(sum((mu_xt-x).^2))/(ss*T));%step1: RMSE of predicted x
A_error = sqrt(sum(sum((A-A_pred).^2))/(ss*ss));% step2: RMSE of predicted A

%% step3: Network refinement
[A_EBCS,sel_ind,A_EBCS_var] = networkRefinement(A_pred,ss,C,os,y,initx,...
    T,mu_xt,noiseR,noiseQ,N_per,T1);
% updated A with the selected time-variant related rows inferred by extended Bayesian compressive sensing
A_EBCS_whole = eye(ss);
A_EBCS_whole(sel_ind,sel_ind) = A_EBCS;
% calculate RMSE of refined A
A_EBCS_error = sqrt(sum(sum((A-A_EBCS_whole).^2))/(ss*ss));

%% return results
result{1} = x;% real x
result{2} = A;% real A
result{3} = mu_xt; % step1: predicted x
result{4} = A_pred; % step2: predicted A
result{5} = x_error; % step1: RMSE of predicted x
result{6} = A_error; % step2: RMSE of predicted A
result{7} = sel_ind; % which row of A is selected in refinement step
result{8} = A_EBCS_whole;%step3: refined A
result{9} = A_EBCS_error;%step3: RMSE of refined A
result{10} = A_EBCS_var;%step3: variance of refined A
